﻿using HotelContracts.BindingModels;
using HotelContracts.SearchModels;
using HotelContracts.StoragesContracts;
using HotelContracts.ViewModels;
using HotelDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HotelDatabaseImplement.Implements
{
    public class RoomStorage : IRoomStorage
    {
        public RoomViewModel? Delete(RoomBindingModel model)
        {
            using var context = new HotelDatabase();
            var element = context.Rooms.FirstOrDefault(x => x.Number == model.Number);
            if (element != null)
            {
                var deletedElement = context.Rooms
                                           .Include(x => x.Worker)
                                           .FirstOrDefault(x => x.Number == model.Number)
                                           ?.GetViewModel;
                context.Rooms.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }

        public RoomViewModel? GetElement(RoomSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Number.ToString()) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new HotelDatabase();
            return context.Rooms.Include(x => x.Worker).FirstOrDefault(x => (!string.IsNullOrEmpty(model.Number.ToString())) && x.Number.ToString() == model.Number.ToString() || model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<RoomViewModel> GetFilteredList(RoomSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Number.ToString()))
            {
                return new();
            }
            using var context = new HotelDatabase();
            return context.Rooms.Include(x => x.Worker).Where(x => x.Number.ToString().Contains(model.Number.ToString())).Select(x => x.GetViewModel).ToList();
        }

        public List<RoomViewModel> GetFullList()
        {
            using var context = new HotelDatabase();
            return context.Rooms.Include(x => x.Worker).Select(x => x.GetViewModel).ToList();
        }

        public RoomViewModel? Insert(RoomBindingModel model)
        {
            var newRoom = Room.Create(model);
            if (newRoom == null)
            {
                return null;
            }
            using var context = new HotelDatabase();
            context.Rooms.Add(newRoom);
            context.SaveChanges();
            return context.Rooms
                      .Include(x => x.Worker)
                      .FirstOrDefault(x => x.Id == newRoom.Id)
                      ?.GetViewModel;
        }

        public RoomViewModel? Update(RoomBindingModel model)
        {
            using var context = new HotelDatabase();
            var component = context.Rooms.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return context.Rooms
                         .Include(x => x.Worker)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }
    }
}
