﻿using HotelContracts.BindingModels;
using HotelContracts.SearchModels;
using HotelContracts.StoragesContracts;
using HotelContracts.ViewModels;
using HotelDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HotelDatabaseImplement.Implements
{
    public class WorkerStorage : IWorkerStorage
    {
        public WorkerViewModel? Delete(WorkerBindingModel model)
        {
            using var context = new HotelDatabase();
            var element = context.Workers.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Workers
                                           .Include(x => x.Post)
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Workers.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }

        public WorkerViewModel? GetElement(WorkerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.FIO) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new HotelDatabase();
            return context.Workers.Include(x => x.Post).FirstOrDefault(x => (!string.IsNullOrEmpty(model.FIO)) && x.FIO == model.FIO || model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }

        public List<WorkerViewModel> GetFilteredList(WorkerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.FIO))
            {
                return new();
            }
            using var context = new HotelDatabase();
            return context.Workers.Include(x => x.Post).Where(x => x.FIO.Contains(model.FIO)).Select(x => x.GetViewModel).ToList();
        }

        public List<WorkerViewModel> GetFullList()
        {
            using var context = new HotelDatabase();
            return context.Workers.Include(x => x.Post).Select(x => x.GetViewModel).ToList();
        }

        public WorkerViewModel? Insert(WorkerBindingModel model)
        {
            var newWorker = Worker.Create(model);
            if (newWorker == null)
            {
                return null;
            }
            using var context = new HotelDatabase();
            context.Workers.Add(newWorker);
            context.SaveChanges();
            return context.Workers
                      .Include(x => x.Post)
                      .FirstOrDefault(x => x.Id == newWorker.Id)
                      ?.GetViewModel;
        }

        public WorkerViewModel? Update(WorkerBindingModel model)
        {
            using var context = new HotelDatabase();
            var component = context.Workers.FirstOrDefault(x => x.Id == model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            context.SaveChanges();
            return context.Workers
                         .Include(x => x.Post)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }
    }
}
